require 'awsprice/version.rb'
require 'json'
require 'net/http'
require 'pstore'

Dir[File.join(File.dirname(__FILE__), 'awsprice/*.rb')].sort.each { |lib| require lib }

module Awsprice
  class Info
    @@REGIONS= {
      'us-east-1' => 'us-east',
      'us-west-1' => 'us-west',
      'us-west-2' => 'us-west-2',
      'eu-west-1' => 'eu-ireland',
      'ap-southeast-1' => 'apac-sin',
      'ap-southeast-2' => 'apac-syd',
      'ap-northeast-1' => 'apac-tokyo',
      'sa-east-1' => 'sa-east-1'
    }


    def initialize
      # @store = PStore.new %(#{ENV['HOME']}/awsprice.pstore)

      # @store[:regions] = []
    end

    def self.fetch_url url
      uri = URI.parse(url)
      page = Net::HTTP.get_response(uri)

      body = page.body.gsub("callback(", "").reverse.sub(")", "").reverse
      if body.split("\n").last == ";"

        body = body.reverse.sub(";", "").reverse
      elsif body[-1] == ";"
        body.chop!
      end

      begin
        JSON.parse(body)
      rescue JSON::ParserError
        JSON.parse(body.gsub(/(\w+)\s*:/, '"\1":'))
      end
    end
  end

  class EC2 < Info
    #attr_accessor :all_instances, :ondemand_instances

    EC2_BASE_URL = "http://a0.awsstatic.com/pricing/1/ec2/"
    # EC2_BASE_URL = "http://aws-assets-pricing-prod.s3.amazonaws.com/pricing/ec2/"

    @@OS_TYPES = [:linux, :mswin, :rhel, :sles, :mswinSQL, :mswinSQLWeb]
    @@RES_TYPES = [:medium, :heavy]

    def initialize
      @ec2_spec =  {}
      @ec2_ondemand_price = []
      @ec2_medium_reserved_price = []
      @ec2_heavy_reserved_price = []
      # unless File.exist? "awsprice.pstore"
      crawl
      # end
    end

    def get_price_reserved_medium
      @ec2_medium_reserved_price
      # @ec2_medium_reserved_price.each do |price|
      #   puts "#{price.region} #{price.size} #{price.os} #{price.yrTerm1} #{price.yrTerm1Hourly}, #{price.yrTerm3}, #{price.yrTerm3Hourly}"
      # end
    end

    def get_price_reserved_heavy
      @ec2_heavy_reserved_price
      # @ec2_heavy_reserved_price.each do |price|
      #   puts "#{price.region} #{price.size} #{price.os} #{price.yrTerm1} #{price.yrTerm1Hourly}, #{price.yrTerm3}, #{price.yrTerm3Hourly}"
      # end
    end

    def get_price_ondemand
      @ec2_ondemand_price
      # @ec2_ondemand_price.each do |price|
      #   puts "#{price.region} #{price.size} #{price.os} #{price.price_per_hour}"
      # end
    end

    def get_spec
      @ec2_spec.values
      # @ec2_spec.each do |key, ins|
      #   puts "#{ins.size} #{ins.type} #{ins.memory}, #{ins.ecu}, #{ins.vcpu}, #{ins.storage}"
      # end
    end

    private

    def crawl
      fetch_ondemand_prices
      fetch_reserved_prices
    end

    def fetch_ondemand_prices
      @@OS_TYPES.each do |os|
        parse_ec2_ondemand_jsons(EC2_BASE_URL + "#{os}-od.min.js")
      end
    end

    def fetch_reserved_prices
      @@OS_TYPES.each do |os|
        @@RES_TYPES.each do |use|
          parse_ec2_reserved_json(EC2_BASE_URL + "#{os}-ri-#{use}.min.js", os, use)
        end
      end
    end

    def parse_ec2_reserved_json url, os, use
      res = Info.fetch_url url
      res['config']['regions'].each do |reg|
        reg['instanceTypes'].each do |type|
          type['sizes'].each do |size|
            # ins_by_region_and_size = instances.select do |ins|
            #   ins.region = reg['region']
            #   ins.size = size['size']
            # end
            price = Price_reserved.new
            price.region = reg['region']
            price.size = size['size']
            price.os = os
            size['valueColumns'].each do |value_col|
              case value_col['name']
              when 'yrTerm1'
                price.yrTerm1 = value_col['prices']['USD']
              when 'yrTerm1Hourly'
                price.yrTerm1Hourly = value_col['prices']['USD']
              when 'yrTerm3'
                price.yrTerm3 = value_col['prices']['USD']
              when 'yrTerm3Hourly'
                price.yrTerm3Hourly = value_col['prices']['USD']
              end
            end

            if use == :medium
              @ec2_medium_reserved_price << price
            elsif use == :heavy
              @ec2_heavy_reserved_price << price
            end


          end
        end
      end
    end

    def parse_ec2_ondemand_jsons url
      # fetch the json object
      res = Info.fetch_url url
      # parse the json object
      res['config']['regions'].each do |reg|
        reg['instanceTypes'].each do |type|
          type['sizes'].each do |size|
            ins = Instance.new
            ins.type = type['type'];
            ins.vcpu = size['vCPU'].to_i
            ins.ecu = size['ECU']
            ins.memory = (size['memoryGiB'].to_f * 1000).to_i
            ins.storage = size['storageGB']
            ins.size = size['size']
            @ec2_spec[size['size']] ||= ins
            size['valueColumns'].each do |value_col|
              price = Price_ondemand.new
              price.region = reg['region']
              price.size = size['size']
              price.os = value_col['name']
              price.price_per_hour = value_col['prices']['USD'].to_f
              @ec2_ondemand_price << price
            end
          end
        end
      end
    end

  end
end
